#ifndef __DXFIMPORT__
#define __DXFIMPORT__

#ifndef __SMKIT__
#include "SMKit.h"
#endif

#ifndef __SMSCENE__
#include "SMScene.h"
#endif

#ifndef __SMFILE__
#include "SMFile.h"
#endif
 
typedef enum {
	kMESH,
	kPOLYLINE,
  kPOLYFACEMESH
} POLY_TYPE;

typedef enum {
    kClose=0x01,
    kCloseV=0x02,
    kCloseH=0x04
} POLY_ATTRIBUTES;

typedef struct PolylineDescription
{
    struct PolylineDescription * next_;
    
    POLY_TYPE type_;
    POLY_ATTRIBUTES attributes_;
    
    unsigned long vertexCount_;
    unsigned long indexedFaceCount_;
    unsigned long h_,v_;

} PolylineDescription;

void getLayerPolyline(SMFile * inFile);
void ExtractPolylineInfo(SMFile *,PolylineDescription *);
void * CreatePolyline(PolylineDescription * );
void FillPolylineStructure(SMFile *,void *);

typedef struct LayerDescription
{
    struct LayerDescription * next_;
    
    unsigned char name_[256];
    unsigned long faceCount_;
    unsigned long lineCount_;
    unsigned long polylineCount_;
    
    PolylineDescription * polylineArray_;
} LayerDescription;

void AddPolyline(LayerDescription *,PolylineDescription *);

typedef struct SceneDescription
{
    unsigned char comments_[256];
    
    unsigned long layerCount_;
    
    LayerDescription * layerArray_;
} SceneDescription;

LayerDescription * FindLayer(SceneDescription *,char *);
void AddLayer(SceneDescription *,LayerDescription *);

SMScene * ImportDXF(char *);

SceneDescription * ExtractFileInfo(SMFile *);
SMScene * CreateScene(SceneDescription *);
void FillSceneStructure(SMScene *,SMFile *);

void * GetNextFreeObject(SMGroup * inGroup);

float readFloat(char *);
void DeleteSceneDescription(SceneDescription *);
void DeleteLayerDescription(LayerDescription *);
void DeletePolylineDescription(PolylineDescription *);

#endif