#import "GraphController.h"
#import "OpenGLView.h"

#include <math.h>
#include "SMKit.h"
#include "DXFImport.h"


@implementation GraphController

- (id) init
{
    [super init];
    
    scene_=NULL;

    return self;
}

- (void) dealloc
{
    

    [super dealloc];
}

- (void) awakeFromNib
{
    [[equationField_ window] makeKeyAndOrderFront:self];
    
    [groupBar_ retain];
    [groupBar_ removeFromSuperview];
    [progressBar_ setUsesThreadedAnimation:YES];
    
    //[self loadScene];
    
    [ graphView_ setScene: scene_];
    //[ graphView_ computeViewDatas];
    //[ graphView_ render];
}

- (void) loadScene
{
    SMFaceArray * tFaceArray;
    SM3DPoint A={{0,0,0},{0,0,0},{0,0}};
    SM3DPoint B={{0,1,0},{0,0,0},{0,0}};
    SM3DPoint C={{1,1,0},{0,0,0},{0,0}};
    SM3DPoint D={{1,0,0},{0,0,0},{0,0}};
        
    SM3DPoint E={{0,0,1},{0,0,0},{0,0}};
    SM3DPoint F={{0,1,1},{0,0,0},{0,0}};
    SM3DPoint G={{1,1,1},{0,0,0},{0,0}};
    SM3DPoint H={{1,0,1},{0,0,0},{0,0}};
    
    
    scene_ = (SMScene *) malloc(sizeof(SMScene));
    scene_->scene_= (SMGroup *) malloc(sizeof(SMGroup));
    scene_->scene_->objectID_=GROUP_ID;
    scene_->scene_->subGroupCount_=1;
    
    scene_->scene_->subGroupArray_=(void **) malloc(1*sizeof(void *));
    
    tFaceArray= (SMFaceArray *) malloc(sizeof(SMFaceArray));
    
    scene_->scene_->subGroupArray_[0]=tFaceArray;
    
    tFaceArray->objectID_=FACE_ARRAY_ID;
    tFaceArray->facesCount_=6;
    
    tFaceArray->array_=(SMFace *) malloc(6*sizeof(SMFace));

    
    tFaceArray->array_[0]=SMCreateSquareFace(&A,&D,&C,&B);
    tFaceArray->array_[1]=SMCreateSquareFace(&D,&H,&G,&C);
    
    tFaceArray->array_[2]=SMCreateSquareFace(&H,&E,&F,&G);
    tFaceArray->array_[3]=SMCreateSquareFace(&E,&A,&B,&F);
    
    tFaceArray->array_[4]=SMCreateSquareFace(&B,&C,&G,&F);
    tFaceArray->array_[5]=SMCreateSquareFace(&E,&H,&D,&A);
    
}

- (void) loadFile:(id) sender
{
    NSOpenPanel *oPanel = [NSOpenPanel openPanel];
    int result;
    
    [oPanel setAllowsMultipleSelection:NO];
    [oPanel setCanChooseDirectories:NO];
    [oPanel setCanChooseFiles:YES];
    
    result = [oPanel runModalForDirectory:nil file:nil types:nil];
    if (result == NSOKButton)
    {
        NSArray *filesToOpen = [oPanel filenames];
        NSString *aFile = [filesToOpen objectAtIndex:0];
        
        // Call the DXF Import code
        
        if (scene_!=NULL)
        {
            SMSceneDelete(scene_);
            [ graphView_ setScene: scene_=NULL];
        }
        
        [equationField_ retain];
        [equationField_ removeFromSuperview];
        
        [bottomGroup_ addSubview:groupBar_ ];
        [groupBar_ release];
        [bottomGroup_ display];
        
        
        [progressBar_ startAnimation:nil];
        
        scene_= ImportDXF([aFile fileSystemRepresentation]);
        
        if (scene_!=NULL)
        {
            
            
            [progressBar_ stopAnimation:nil];
            
            [groupBar_ removeFromSuperview];
            [groupBar_ retain];
            [bottomGroup_ addSubview:equationField_ ];
            [equationField_ release];
            
            [equationField_ setStringValue:aFile];
            [bottomGroup_ display];
            
            [ graphView_ setScene: scene_];
            [ graphView_ computeViewDatas];
            [ graphView_ render];
        }
        else
        {
            [progressBar_ stopAnimation:nil];
            
            [groupBar_ retain];
            [groupBar_ removeFromSuperview];
            [bottomGroup_ addSubview:equationField_ ];
            [equationField_ release];
            
            [equationField_ setStringValue:@""];
            [bottomGroup_ display];
            
            NSRunAlertPanel(@"Import failed", @"The import process has failed.\nThis file is not a DXF file or its pathanme is incorrect.\nCheck that the path of the file does not contain any forbiden characters such as  or .", nil, nil, nil);
        }
    }
}

- (BOOL)application:(NSApplication *)theApplication openFile:(NSString *)filename
{
    if (scene_!=NULL)
    {
        SMSceneDelete(scene_);
        [ graphView_ setScene: scene_=NULL];
    }
    
    [equationField_ retain];
    [equationField_ removeFromSuperview];
    [bottomGroup_ addSubview:groupBar_ ];
    [groupBar_ release];
    [bottomGroup_ display];
    
    [progressBar_ startAnimation:nil];
    
    scene_= ImportDXF([filename fileSystemRepresentation]);
    
    if (scene_!=NULL)
    {
                
        [progressBar_ stopAnimation:nil];
        
        [groupBar_ retain];
        [groupBar_ removeFromSuperview];
        [bottomGroup_ addSubview:equationField_ ];
        [equationField_ release];
        
        [equationField_ setStringValue:filename];
        [bottomGroup_ display];
        
        [ graphView_ setScene: scene_];
        [ graphView_ computeViewDatas];
        [ graphView_ render];

    }
    else
    {
        [progressBar_ stopAnimation:nil];
        
        [groupBar_ retain];
        [groupBar_ removeFromSuperview];
        [bottomGroup_ addSubview:equationField_ ];
        [equationField_ release];
        
        [equationField_ setStringValue:@""];
        [bottomGroup_ display];
        NSRunAlertPanel(@"Import failed", @"The import process has failed.\nThis file is not a DXF file or its pathanme is incorrect.\nCheck that the path of the file does not contain any forbiden characters such as  or .", nil, nil, nil);
    }
    
    return YES;
}

- (BOOL)windowShouldClose:(id)sender
{
    [[NSApplication sharedApplication] terminate:self];
    
    return YES;
}

@end
