#import "OpenGLView.h"
#import <OpenGL/gl.h>
#import <OpenGL/glu.h>

#include "SMDrawingEngine.h"
#include "SMBounding.h"
#include "SMNormalEngine.h"

#define __DRAWNORMALS	0
#define __DEPTH_BUFFER	1

#define KEY_ROTATION	9

@implementation OpenGLView

- (id) initWithFrame:(NSRect) frameRect
{

    NSOpenGLPixelFormatAttribute attributes[]={
        NSOpenGLPFADepthSize,16,
        //NSOpenGLPFAMultiScreen,
        NSOpenGLPFAMinimumPolicy,
        NSOpenGLPFAClosestPolicy,
        0
    };

    NSOpenGLPixelFormat * PixelFormat=[[NSOpenGLPixelFormat alloc] initWithAttributes:attributes];

    wireFrame_=YES;
    shadingMode_=kSMAqua;

    timer_=NULL;
    
    showAxis_=NO;
    showFog_=NO;

    zoom_=0;
    
    scene_=NULL;

    rotY_=rotX_=rotZ_=0;

    needDisplay_=false;
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(shapeView) name:NSViewFrameDidChangeNotification object:nil];
    return [super initWithFrame: frameRect pixelFormat:PixelFormat];
}

- (void) dealloc
{
    if (timer_!=NULL)
    {
        [timer_ invalidate];
        [timer_ release];
        timer_=NULL;
    }
    
    

    [super dealloc];
}

- (void) awakeFromNib
{
    [self shapeView];

    [self setNeedsDisplay: YES];
}

- (SMScene *) scene
{
    return scene_;
}

- (void) setScene:(SMScene *) inScene
{
    scene_=inScene;
}

- (void) setShadeMode:(unsigned short) inShadingMode
{
    shadingMode_=inShadingMode;
}

- (void) setDrawingModeWireFrame:(BOOL) inWireFrame
{
    wireFrame_=inWireFrame;
}

- (void) computeViewDatas
{
    float tscale;
    
    scene_->min_.x=32535;
    scene_->min_.y=32535;
    scene_->min_.z=32535;
    
    scene_->max_.x=-32535;
    scene_->max_.y=-32535;
    scene_->max_.z=-32535;
    
    SMSceneGetMinMax(scene_);
    SMSceneNormal(scene_);
    
    centerX_=(scene_->min_.x+scene_->max_.x)*0.5;
    centerY_=(scene_->min_.y+scene_->max_.y)*0.5;
    centerZ_=(scene_->min_.z+scene_->max_.z)*0.5;

    scale_=50.0/(scene_->max_.x-scene_->min_.x);
    tscale=50.0/(scene_->max_.y-scene_->min_.y);

    if (tscale<scale_) scale_=tscale;

    tscale=50.0/(scene_->max_.z-scene_->min_.z);

    if (tscale<scale_) scale_=tscale;
}

- (void) manageLights
{
    if (wireFrame_==NO)
    {
        GLfloat pos[]={centerX_,centerZ_+(scene_->max_.z-scene_->min_.z)*10,centerY_,0.0};
        GLfloat color[]={0.3,0.3,0.3,1.0};
        GLfloat ambient[]={0.2,0.2,0.2,1.0};
        GLfloat specular[]={1.0,1.0,1,1};
        GLfloat spot_direction[]={0,-1,0,1};
       //GLfloat shininess[]={50};

        glEnable(GL_LIGHTING);
        //glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);
        
        glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambient);
        
        glLightfv(GL_LIGHT0,GL_DIFFUSE,color);
        //glLightfv(GL_LIGHT0,GL_AMBIENT,ambient);
        //glLightfv(GL_LIGHT0,GL_SPECULAR,specular);
        glLightfv(GL_LIGHT0,GL_POSITION,pos);
        //glLightf(GL_LIGHT0,GL_SPOT_CUTOFF,45.0);
        //glLightfv(GL_LIGHT0,GL_SPOT_DIRECTION,spot_direction);
        //glLightModeli(GL_LIGHT_MODEL_TWO_SIDE,0);
        //glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE,specular);
        //glMaterialfv(GL_FRONT,GL_SHININESS,shininess);
        glEnable(GL_LIGHT0);
        glEnable(GL_COLOR_MATERIAL);
    }
    else
    {
        glDisable(GL_COLOR_MATERIAL);
        glDisable(GL_LIGHT0);
        glDisable(GL_LIGHTING);
        
    }
    
    
}

- (void) drawRect:(NSRect) rect
{
    [[self openGLContext] makeCurrentContext];
    
    glClearColor(0,0,0,0);

    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

    
    //[self manageLights];
    
    if (needDisplay_==YES && scene_!=NULL)
    {
        /*glEnable(GL_CULL_FACE);
        glCullFace(GL_BACK);*/

        glLoadIdentity();
        
        glTranslatef(0,0,zoom_-100);
            
        
            
        if (showFog_==YES)
        {
            
            
            glEnable(GL_FOG);
            
            //glFogf(GL_FOG_DENSITY, 0.35);
            glFogf(GL_FOG_START,scene_->min_.x/scale_);
            glFogf(GL_FOG_END,centerX_/scale_);
            
            //glFogf(GL_FOG_DENSITY,0.35);
            
            
            
        }
        else
        {
            glDisable(GL_FOG);
        }
        
        glPushMatrix();

        glRotatef(rotX_,1.0f,0.0f,0.0f);
        glRotatef(rotY_,0.0f,1.0f,0.0f);

        glScalef(scale_,scale_,scale_);
            
        glTranslatef(-centerX_,-centerZ_,-centerY_);
        
        [self manageLights];
        
        if (showAxis_==YES)
        {
            float xDiffuse[]={1,0,0,1};
            float yDiffuse[]={0,1,0,1};
            float zDiffuse[]={0,0,1,1};
            float myScale=50.0/scale_;
            float ratio;
            
            glShadeModel(GL_FLAT);
            glPolygonMode(GL_FRONT,GL_FILL);
            glPolygonMode(GL_BACK, GL_FILL);
            
            glLineWidth(2);
            glBegin(GL_LINES);
            
            if (wireFrame_==YES) glColor4fv(xDiffuse);
            else
            {
                glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE,xDiffuse);
            }
            glVertex3f(centerX_,centerZ_,centerY_);
            
            if (scene_->max_.x>0) ratio=1.25;
            else ratio=0.75;
            
            glVertex3f(scene_->max_.x*ratio,centerZ_,centerY_);
            
            if (wireFrame_==YES) glColor4fv(yDiffuse);
            else
            {
                glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE,yDiffuse);
            }

            if (scene_->max_.y>0) ratio=1.25;
            else ratio=0.75;
            
            glVertex3f(centerX_,centerZ_,centerY_);
            glVertex3f(centerX_,centerZ_,scene_->max_.y*ratio);
            
            if (wireFrame_==YES) glColor4fv(zDiffuse);
            else
            {
                glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE,zDiffuse);
            }

            if (scene_->max_.z>0) ratio=1.25;
            else ratio=0.75;
            
            glVertex3f(centerX_,centerZ_,centerY_);
            glVertex3f(centerX_,scene_->max_.z*ratio,centerY_);
            glEnd();
            glLineWidth(1);
            
            glBegin(GL_TRIANGLES);
            
            if (wireFrame_==YES) glColor4fv(xDiffuse);
            else
            {
                glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE,xDiffuse);
            }
            myScale*=0.05;
            
            if (scene_->max_.x>0) ratio=1.25;
            else ratio=0.75;
            
            glVertex3f(scene_->max_.x*ratio+myScale,centerZ_,centerY_);
            glVertex3f(scene_->max_.x*ratio,centerZ_,centerY_+myScale*0.75);
            glVertex3f(scene_->max_.x*ratio,centerZ_,centerY_-myScale*0.75);
            
            if (wireFrame_==YES) glColor4fv(yDiffuse);
            else
            {
                glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE,yDiffuse);
            }
            
            if (scene_->max_.y>0) ratio=1.25;
            else ratio=0.75;

            glVertex3f(centerX_,centerZ_,scene_->max_.y*ratio+myScale);
            glVertex3f(centerX_+myScale*0.75,centerZ_,scene_->max_.y*ratio);
            glVertex3f(centerX_-myScale*0.75,centerZ_,scene_->max_.y*ratio);
            
            if (wireFrame_==YES) glColor4fv(zDiffuse);
            else
            {
                glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE,zDiffuse);
            }

            if (scene_->max_.z>0) ratio=1.25;
            else ratio=0.75;
            
            glVertex3f(centerX_,scene_->max_.z*ratio+myScale,centerY_);
            glVertex3f(centerX_+myScale*0.5,scene_->max_.z*ratio,centerY_);
            glVertex3f(centerX_-myScale*0.5,scene_->max_.z*ratio,centerY_);
            
            glVertex3f(centerX_,scene_->max_.z*ratio+myScale,centerY_);
            glVertex3f(centerX_,scene_->max_.z*ratio,centerY_-myScale*0.5);
            glVertex3f(centerX_,scene_->max_.z*ratio,centerY_+myScale*0.5);
            
            glEnd();
        }
        
        if (wireFrame_==YES)
        {
            glPolygonMode(GL_FRONT,GL_LINE);
            glPolygonMode(GL_BACK, GL_LINE);
            glShadeModel(GL_FLAT);
        }
        else
        {
            glPolygonMode(GL_FRONT,GL_FILL);
            glPolygonMode(GL_BACK, GL_FILL);
            glShadeModel(GL_SMOOTH);
        }

       	SMSceneDraw(scene_);

        glPopMatrix();

        needDisplay_=YES;
    }

    glFinish();

    [[self openGLContext] flushBuffer];
}

- (void)render
{
    needDisplay_=YES;
    [self setNeedsDisplay:TRUE];
}

- (void)reshape
{
     NSRect bounds  =[self bounds];
     NSRect visRect =[self visibleRect];

     [[self openGLContext] makeCurrentContext];
     glViewport(-floor(visRect.origin.x), -floor(visRect.origin.y),  floor(bounds.size.width),  floor(bounds.size.height));
     glScissor(0, 0, floor(visRect.size.width), floor(visRect.size.height));
}

- (void) shapeView
{
    GLfloat fogColor[4]={0.0,0.0,0.0,1};
    
    [[self openGLContext] makeCurrentContext];
    
    //[super update];
    
    //glViewport(0,0,_frame.size.width,_frame.size.height);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(60.0f,_frame.size.width/_frame.size.height,0.1,1000);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

//#if __DEPTH_BUFFER
    glDepthFunc(GL_LESS);
    glEnable(GL_DEPTH_TEST);
    //glFrontFace(GL_CCW);
    //glClearDepth(1);
    
    //
//#endif    
    /*glFogi(GL_FOG_MODE,GL_LINEAR);
    glFogi(GL_FOG_HINT,GL_NICEST);
    glFogfv(GL_FOG_COLOR,fogColor);*/

}

- (BOOL)acceptsFirstResponder
{
    return YES;
}

- (BOOL)acceptsFirstMouse:(NSEvent *)theEvent
{
    return YES;
}

- (void)mouseDown:(NSEvent *)theEvent
{

    BOOL keepOn = YES;
    NSPoint mouseLoc;
    NSPoint oldMouseLoc=[self convertPoint:[theEvent locationInWindow] fromView:nil];

    deltaRotY_=0;
    do {
        mouseLoc = [self convertPoint:[theEvent locationInWindow] fromView:nil];

        switch ([theEvent type]) {
            case NSLeftMouseDragged:
                if (([theEvent modifierFlags] & NSCommandKeyMask) !=0)
                {
                    zoom_-=(mouseLoc.y-oldMouseLoc.y)*100.0/_frame.size.height;
                    if (zoom_>100) zoom_=100;
                }
                else
                {
                    rotX_-=(mouseLoc.y-oldMouseLoc.y)*360.0/_frame.size.height;

                    if (rotX_<0) rotX_+=360;
                    if (rotX_>360) rotX_-=360;

                    
                    rotY_+=(deltaRotY_=(mouseLoc.x-oldMouseLoc.x)*360.0/_frame.size.width);
                    if (rotY_<0) rotY_+=360;
                    if (rotY_>360) rotY_-=360;
                }
                [self render];

                oldMouseLoc=mouseLoc;
                break;
            case NSLeftMouseUp:
                if (timer_!=NULL)
                {
                    [timer_ invalidate];
                    [timer_ release];
                    timer_=NULL;
                }

                if (deltaRotY_!=0)
                {
                    timer_ = [[NSTimer scheduledTimerWithTimeInterval:0.05 target:
                                                                self selector:@selector(animate:)
                                                                     userInfo:nil
                                                                      repeats:YES] retain];
                }
                keepOn = NO;
                return;
            default:
                /* Ignore any other kind of event. */
                break;
        }

        theEvent = [[self window] nextEventMatchingMask: NSLeftMouseUpMask |
                   NSLeftMouseDraggedMask];

    } while (keepOn);

}

- (void) keyDown:(NSEvent *) theEvent
{
    NSString * tString;
    unsigned int stringLength;
    unsigned int i;
    unichar tChar;
    
    tString= [theEvent characters];
    
    stringLength=[tString length];
    
    for(i=0;i<stringLength;i++)
    {
        tChar=[tString characterAtIndex:i];
        
        switch(tChar)
        {
            case NSUpArrowFunctionKey:
                rotX_-=KEY_ROTATION;
                if (rotX_<0) rotX_+=360;
                if (rotX_>360) rotX_-=360;

                [self render];
                break;
            case NSDownArrowFunctionKey:
                rotX_+=KEY_ROTATION;
                if (rotX_<0) rotX_+=360;
                if (rotX_>360) rotX_-=360;

                [self render];
                break;
            case NSLeftArrowFunctionKey:
                if (timer_!=NULL)
                {
                    [timer_ invalidate];
                    [timer_ release];
                    timer_=NULL;
                }
                rotY_-=KEY_ROTATION;
                if (rotY_<0) rotY_+=360;
                if (rotY_>360) rotY_-=360;

                [self render];
                break;
            case NSRightArrowFunctionKey:
                if (timer_!=NULL)
                {
                    [timer_ invalidate];
                    [timer_ release];
                    timer_=NULL;
                }
                rotY_+=KEY_ROTATION;
                if (rotY_<0) rotY_+=360;
                if (rotY_>360) rotY_-=360;

                [self render];
                break;
            case NSF1FunctionKey:
                wireFrame_=!wireFrame_;
                
                if (wireFrame_==YES) SetRenderingMode(SM_WIREFRAME);
                else SetRenderingMode(SM_FILLED);
                [self render];
                break;
            case NSF2FunctionKey:
                showAxis_=!showAxis_;
                [self render];
                break;
            case NSF3FunctionKey:
                showFog_=!showFog_;
                [self render];
                break;
            case NSF4FunctionKey:
                SetColoringMode(SM_LAYER_COLOR-GetColoringMode());
                [self render];
                break;
                
        }
    }
}

- (void) animate: (id) sender
{
    rotY_+=deltaRotY_;
    if (rotY_<0) rotY_+=360;
    if (rotY_>360) rotY_-=360;

    [self render];

}

@end
