#include "SMDrawingEngine.h"
#include "SMKit.h"
#include <stdlib.h>
#include <OpenGL/gl.h>
#include <OpenGL/glu.h>

SMDrawingEngine gDrawingEngine;

void InitDrawingEngine(void)
{
    unsigned long i=0;
    
    gDrawingEngine.drawingCallBackCount_=SMSD_TYPEDOBJECT_COUNT;
    
    gDrawingEngine.array_= (SMDraw *) malloc(SMSD_TYPEDOBJECT_COUNT*sizeof(SMDraw));
    
    gDrawingEngine.array_[i++]=SMGroupDraw;
    gDrawingEngine.array_[i++]=SMFaceArrayDraw;
    gDrawingEngine.array_[i++]=SMGridDraw;
    gDrawingEngine.array_[i++]=SMPolygonDraw;
    gDrawingEngine.array_[i++]=SMMeshDraw;
    gDrawingEngine.array_[i++]=SMLineArrayDraw;
    
    gDrawingEngine.renderingMode_=SM_WIREFRAME;
    gDrawingEngine.coloringMode_=SM_LAYER_COLOR;
}

void SetRenderingMode(int inMode)
{
    gDrawingEngine.renderingMode_=inMode;
}

void SetColoringMode(int inMode)
{
    gDrawingEngine.coloringMode_=inMode;
}

int GetColoringMode(void)
{
    return gDrawingEngine.coloringMode_;
}

void SMSceneDraw(SMScene * inScene)
{
    if (gDrawingEngine.coloringMode_==SM_SCENE_COLOR)
    {
        if (gDrawingEngine.renderingMode_==SM_WIREFRAME)
            {
            glColor4fv((float *)&inScene->ambient_);
        }
        else
        {
            glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT,(float *)&inScene->ambient_);
            glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,(float *)&inScene->diffuse_);
        }
    }
    
    SMGroupDraw(inScene->scene_);
}

void SMGroupDraw(void * inGroup)
{
    SMGroup * tGroup=(SMGroup *) inGroup;
    unsigned long i;
    //float diffuse[]={0.6,0.6,0.6,1.0};
    //float ambient[]={0.9,0.9,0.9,1.0};
    
    for(i=0;i<tGroup->subGroupCount_;i++)
    {
        if (gDrawingEngine.coloringMode_==SM_LAYER_COLOR)
        {
            if (gDrawingEngine.renderingMode_==SM_WIREFRAME)
            {
                glColor4fv((float *)&tGroup->ambient_);
            }
            else
            {
                glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT,(float *)&tGroup->ambient_);
                glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,(float *)&tGroup->diffuse_);
                //glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,(float *)&tGroup->diffuse_);
            }
        }
        
        gDrawingEngine.array_[*((unsigned long *) tGroup->subGroupArray_[i])](tGroup->subGroupArray_[i]);
    }
}

void SMFaceArrayDraw(void * inFaceArray)
{
    SMFaceArray * tFaceArray=(SMFaceArray *) inFaceArray;
    unsigned long i;
    
    glBegin(GL_TRIANGLES);
    
    for(i=0;i<tFaceArray->facesCount_;i++)
    {
        SMFace * tFace=tFaceArray->array_+i;
        
        glNormal3f(tFace->normal_.x,tFace->normal_.z,tFace->normal_.y);
        
        if (tFace->verticesCount_==3)
        {
            glVertex3f(tFace->vertices_[0].position_.x,tFace->vertices_[0].position_.z,tFace->vertices_[0].position_.y);
            glVertex3f(tFace->vertices_[1].position_.x,tFace->vertices_[1].position_.z,tFace->vertices_[1].position_.y);
            glVertex3f(tFace->vertices_[2].position_.x,tFace->vertices_[2].position_.z,tFace->vertices_[2].position_.y);
        }
        else
        {
            
            glVertex3f(tFace->vertices_[0].position_.x,tFace->vertices_[0].position_.z,tFace->vertices_[0].position_.y);
            glVertex3f(tFace->vertices_[1].position_.x,tFace->vertices_[1].position_.z,tFace->vertices_[1].position_.y);
            glVertex3f(tFace->vertices_[2].position_.x,tFace->vertices_[2].position_.z,tFace->vertices_[2].position_.y);
            
            glVertex3f(tFace->vertices_[2].position_.x,tFace->vertices_[2].position_.z,tFace->vertices_[2].position_.y);
            glVertex3f(tFace->vertices_[3].position_.x,tFace->vertices_[3].position_.z,tFace->vertices_[3].position_.y);
            glVertex3f(tFace->vertices_[0].position_.x,tFace->vertices_[0].position_.z,tFace->vertices_[0].position_.y);
        }
    }
    
    glEnd();
    
    /*glBegin(GL_LINES);	// To draw the normals

    glColor3f(1.0f,0.0f,0.0f);

    for(i=0;i<tFaceArray->facesCount_;i++)
    {
        SMFace * tFace=tFaceArray->array_+i;
        int j;
        
        for(j=0;j<tFace->verticesCount_;j++)
        {
            glVertex3f(tFace->vertices_[j].position_.x,tFace->vertices_[j].position_.z,tFace->vertices_[j].position_.y);
            glVertex3f(tFace->vertices_[j].position_.x+tFace->normal_.x,tFace->vertices_[j].position_.z+tFace->normal_.z,tFace->vertices_[j].position_.y+tFace->normal_.y);
        }
    }
    glEnd();*/

}

void SMGridDraw(void * inGrid)
{
    SMGrid * tGrid=(SMGrid *) inGrid;
    unsigned long i,j;
    SM3DPoint * tArray=tGrid->array_;
    unsigned long maxStrip=tGrid->verticesCount_-tGrid->hVerticesCount_;
    register unsigned long buffhVerticesCount_=tGrid->hVerticesCount_;
    
    for(i=0;i<maxStrip;i+=buffhVerticesCount_)
    {
        glBegin(GL_QUAD_STRIP);
        
        for(j=i;j<(i+buffhVerticesCount_);j++)
        {
           glNormal3f(tArray[j].normal_.x,tArray[j].normal_.z,tArray[j].normal_.y);
           glVertex3f(tArray[j].position_.x,tArray[j].position_.z,tArray[j].position_.y);
           glNormal3f(tArray[j+buffhVerticesCount_].position_.x,tArray[j].position_.z,tArray[j+buffhVerticesCount_].position_.y);
           glVertex3f(tArray[j+buffhVerticesCount_].normal_.x,tArray[j+buffhVerticesCount_].normal_.z,tArray[j+buffhVerticesCount_].normal_.y);
        }
                
        glEnd();
    }
}

void SMPolygonDraw(void * inPolygon)
{
    SMPolygon * tPolygon=(SMPolygon *) inPolygon;
    unsigned long i;
    
    if (tPolygon->type_==0)
    {
        glBegin(GL_LINE_STRIP);
        
        for(i=0;i<tPolygon->verticesCount_;i++)
        {
            glVertex3f(tPolygon->vertices_[i].position_.x,tPolygon->vertices_[i].position_.z,tPolygon->vertices_[i].position_.y);
        }
        
        glEnd();
    }
    else
    {
    }
}

void SMMeshDraw(void * inMesh)
{
    SMMesh * tMesh=(SMMesh *) inMesh;
    unsigned long i;
    SMIndexedFace * tArrayFace=tMesh->arrayFace_;
    SM3DPoint * tArrayPoint=tMesh->arrayPoint_;
    unsigned long i1,i2,i3,i4;
    
    glBegin(GL_TRIANGLES);
    
    for(i=0;i<tMesh->facesCount_;i++)
    {
        i1=tArrayFace[i].index_[0];
        i2=tArrayFace[i].index_[1];
        i3=tArrayFace[i].index_[2];
        i4=tArrayFace[i].index_[3];
        
        if (i3==i4)
        {
            glNormal3f(tArrayPoint[i1].normal_.x,tArrayPoint[i1].normal_.z,tArrayPoint[i1].normal_.y);
            glVertex3f(tArrayPoint[i1].position_.x,tArrayPoint[i1].position_.z,tArrayPoint[i1].position_.y);
            glNormal3f(tArrayPoint[i2].normal_.x,tArrayPoint[i2].normal_.z,tArrayPoint[i2].normal_.y);
            glVertex3f(tArrayPoint[i2].position_.x,tArrayPoint[i2].position_.z,tArrayPoint[i2].position_.y);
            glNormal3f(tArrayPoint[i3].normal_.x,tArrayPoint[i3].normal_.z,tArrayPoint[i3].normal_.y);
            glVertex3f(tArrayPoint[i3].position_.x,tArrayPoint[i3].position_.z,tArrayPoint[i3].position_.y);
        }
        else
        {
            glNormal3f(tArrayPoint[i1].normal_.x,tArrayPoint[i1].normal_.z,tArrayPoint[i1].normal_.y);
            glVertex3f(tArrayPoint[i1].position_.x,tArrayPoint[i1].position_.z,tArrayPoint[i1].position_.y);
            glNormal3f(tArrayPoint[i2].normal_.x,tArrayPoint[i2].normal_.z,tArrayPoint[i2].normal_.y);
            glVertex3f(tArrayPoint[i2].position_.x,tArrayPoint[i2].position_.z,tArrayPoint[i2].position_.y);
            glNormal3f(tArrayPoint[i3].normal_.x,tArrayPoint[i3].normal_.z,tArrayPoint[i3].normal_.y);
            glVertex3f(tArrayPoint[i3].position_.x,tArrayPoint[i3].position_.z,tArrayPoint[i3].position_.y);
            
            glNormal3f(tArrayPoint[i3].normal_.x,tArrayPoint[i3].normal_.z,tArrayPoint[i3].normal_.y);
            glVertex3f(tArrayPoint[i3].position_.x,tArrayPoint[i3].position_.z,tArrayPoint[i3].position_.y);
            glNormal3f(tArrayPoint[i4].normal_.x,tArrayPoint[i4].normal_.z,tArrayPoint[i4].normal_.y);
            glVertex3f(tArrayPoint[i4].position_.x,tArrayPoint[i4].position_.z,tArrayPoint[i4].position_.y);
            glNormal3f(tArrayPoint[i1].normal_.x,tArrayPoint[i1].normal_.z,tArrayPoint[i1].normal_.y);
            glVertex3f(tArrayPoint[i1].position_.x,tArrayPoint[i1].position_.z,tArrayPoint[i1].position_.y);
        }
    }
    
    glEnd();
}

void SMLineArrayDraw(void * inLineArray)
{
    SMLineArray * tLineArray=(SMLineArray *) inLineArray;
    SMLine * tArray=tLineArray->array_;
    unsigned long i;
    
    glBegin(GL_LINES);
    
    for(i=0;i<tLineArray->linesCount_;i++)
    {
        glVertex3f(tArray[i].a_.position_.x,tArray[i].a_.position_.z,tArray[i].a_.position_.y);
        glVertex3f(tArray[i].b_.position_.x,tArray[i].b_.position_.z,tArray[i].b_.position_.y);
    }
    
    glEnd();
}