#include "SMNormalEngine.h"
#include "SMKit.h"
#include <stdlib.h>

SMNormalEngine gNormalEngine;

void InitNormalEngine(void)
{
    unsigned long i=0;
    
    gNormalEngine.normalCallBackCount_=SMSD_TYPEDOBJECT_COUNT;
    
    gNormalEngine.array_= (SMNormal *) malloc(SMSD_TYPEDOBJECT_COUNT*sizeof(SMNormal));
    
    gNormalEngine.array_[i++]=SMGroupNormal;
    gNormalEngine.array_[i++]=SMFaceArrayNormal;
    gNormalEngine.array_[i++]=SMGridNormal;
    gNormalEngine.array_[i++]=SMPolygonNormal;
    gNormalEngine.array_[i++]=SMMeshNormal;
    gNormalEngine.array_[i++]=SMLineArrayNormal;
}

void SMSceneNormal(SMScene * inScene)
{
    SMGroupNormal(inScene->scene_);
}

void SMGroupNormal(void * inGroup)
{
    SMGroup * tGroup=(SMGroup *) inGroup;
    unsigned long i;
    
    for(i=0;i<tGroup->subGroupCount_;i++)
    {
        gNormalEngine.array_[*((unsigned long *) tGroup->subGroupArray_[i])](tGroup->subGroupArray_[i]);
    }
}

void SMFaceArrayNormal(void * inFaceArray)
{
    SMFaceArray * tFaceArray=(SMFaceArray *) inFaceArray;
    unsigned long i;
    
    for(i=0;i<tFaceArray->facesCount_;i++)
    {
        SMFace * tFace=tFaceArray->array_+i;
        
        tFace->normal_=SMPointComputeNormal(tFace->vertices_[0].position_,tFace->vertices_[1].position_,tFace->vertices_[2].position_);
    }
}

void SMGridNormal(void * inGrid)
{
    SMGrid * tGrid=(SMGrid *) inGrid;
    unsigned long i,j;
    SM3DPoint * tArray=tGrid->array_;
    register unsigned long buffhVerticesCount_=tGrid->hVerticesCount_;
    unsigned long maxStrip=tGrid->verticesCount_-buffhVerticesCount_;
    SMNormalVector tNormal;
    
    
    for(i=0;i<maxStrip;i+=buffhVerticesCount_)
    {
        for(j=i;j<(i+buffhVerticesCount_-1);j++)
        {
           tNormal=SMPointComputeNormal(tArray[j].position_,tArray[j+buffhVerticesCount_].position_,tArray[j+buffhVerticesCount_+1].position_);
           
           tArray[j].normal_=AddVector(tArray[j].normal_,tNormal);
           tArray[j+buffhVerticesCount_].normal_=AddVector(tArray[j+buffhVerticesCount_].normal_,tNormal);
           tArray[j+buffhVerticesCount_+1].normal_=AddVector(tArray[j+buffhVerticesCount_+1].normal_,tNormal);
        }
    }
    
    for(i=0;i<tGrid->verticesCount_;i++)
    {
        tArray[i].normal_=NormalizeVector(tArray[i].normal_);
    }
}

void SMPolygonNormal(void * inPolygon)
{
    SMPolygon * tPolygon=(SMPolygon *) inPolygon;
    
    if (tPolygon->type_==0)
    {
        // No need to compute normals on LINE_STRIP
    }
    else
    {
        tPolygon->normal_=SMPointComputeNormal(tPolygon->vertices_[0].position_,tPolygon->vertices_[1].position_,tPolygon->vertices_[2].position_);
    }
}

void SMMeshNormal(void * inMesh)
{
    SMMesh * tMesh=(SMMesh *) inMesh;
    unsigned long i;
    SMIndexedFace * tArrayFace=tMesh->arrayFace_;
    SM3DPoint * tArrayPoint=tMesh->arrayPoint_;
    unsigned long i1,i2,i3,i4;
    SMNormalVector tNormal;
    
    for(i=0;i<tMesh->facesCount_;i++)
    {
        i1=tArrayFace[i].index_[0];
        i2=tArrayFace[i].index_[1];
        i3=tArrayFace[i].index_[2];
        i4=tArrayFace[i].index_[3];
        
        tNormal=SMPointComputeNormal(tArrayPoint[i1].position_,tArrayPoint[i2].position_,tArrayPoint[i3].position_);
        
        tArrayPoint[i1].normal_=AddVector(tArrayPoint[i1].normal_,tNormal);
        tArrayPoint[i2].normal_=AddVector(tArrayPoint[i2].normal_,tNormal);
        tArrayPoint[i3].normal_=AddVector(tArrayPoint[i3].normal_,tNormal);
        
        if (i3!=i4)
        {
            tArrayPoint[i4].normal_=AddVector(tArrayPoint[i4].normal_,tNormal);
            
        }
    }
    
    for(i=0;i<tMesh->verticesCount_;i++)
    {
        tArrayPoint[i].normal_=NormalizeVector(tArrayPoint[i].normal_);
    }
}

void SMLineArrayNormal(void * inLineArray)
{
    // No need to compute normals on LINE
}